import { CSSProperties } from 'vue'
import { CommonData, MaybeRef } from '~/common'

interface TrackNode {
  name: string
  nodeCode: string
  flag: boolean
  labelWidth?: number
  descriptions: string[]
}
interface TrackAirport {
  airport: string
  isDeparture: boolean
  trackSteps: TrackNode[]
}
interface TrackAirline {
  flightNO: string
  flightDate: string
  airports: TrackAirport[]
}

const trackNodesMap = {
  departure: [
    {
      name: '收货核单',
      nodeCode: 'DEH',
    },
    {
      name: '查验',
      nodeCode: 'ACC_CHECK',
    },
    {
      name: '安检',
      nodeCode: '安检', // 暂无
    },
    {
      name: '加货',
      nodeCode: 'ACC_BUP',
    },
    {
      name: '待运区',
      nodeCode: 'WAT_LOC',
    },
    {
      name: '货站交接',
      nodeCode: 'CARGOS_DEP_ULD_HANDOVER',
    },
    {
      name: '机下交接',
      nodeCode: '出港货邮',
    },
    {
      name: '装机',
      nodeCode: '装载完成',
    },
    {
      name: '拉下',
      nodeCode: 'CARGOS_OFFLOAD',
    },
    {
      name: '拉回',
      nodeCode: 'OFFLOAD_CONFIRM',
    },
    // {
    //   name: '起飞',
    //   nodeCode: 'TAKEOFF', // 待定
    // },
    {
      name: '退运',
      nodeCode: 'BILL_RETURN',
    },
  ],
  arrival: [
    {
      name: '卸机',
      nodeCode: 'CARGOS_ARR_HANDOVER',
    },
    {
      name: '机下交接',
      nodeCode: 'CARGOS_HANDOVER_STATUS',
    },
    {
      name: '货站交接',
      nodeCode: '货站交接', // 暂无
    },
    {
      name: '理货',
      nodeCode: 'IMP_TALLY',
    },
    {
      name: '出库',
      nodeCode: 'FSU_DLV',
    },
  ],
  internationalDeparture: [],
  internationalArrival: [],
}

export default function useTrackData(
  name: string,
  trackData: MaybeRef<CommonData[]>
) {
  const isInternational = name.includes('International')
  const trackAirlines = ref<TrackAirline[]>([])

  const getTrackAirlines = () => {
    const airlines = unref(trackData).reduce(
      (
        airlines,
        {
          flightNO,
          flightDate,
          LoadingLocation,
          UnloadingLocation,
          nodeCode,
          execPosition,
          ConsignmentItemPackagingQuantityQuantity,
          execResult,
          execTime,
        }
      ) => {
        const isDeparture = [
          'DEH',
          'ACC_CHECK',
          '安检',
          'ACC_BUP',
          'WAT_LOC',
          'CARGOS_DEP_ULD_HANDOVER',
          '出港货邮',
          '装载完成',
          'CARGOS_OFFLOAD',
          'OFFLOAD_CONFIRM',
          'BILL_RETURN',
        ].includes(String(nodeCode))
        const airport = isDeparture ? LoadingLocation : UnloadingLocation
        const trackNode = {
          flag: Boolean(
            execPosition ||
              ConsignmentItemPackagingQuantityQuantity ||
              execResult ||
              execTime
          ),
          descriptions: [
            execPosition,
            ConsignmentItemPackagingQuantityQuantity,
            execResult,
            String(execTime ?? '').split('T')[1],
          ],
        }
        const nodeList = trackNodesMap[
          isDeparture
            ? isInternational
              ? 'internationalDeparture'
              : 'departure'
            : isInternational
            ? 'internationalArrival'
            : 'arrival'
        ].map(node => {
          if (node.nodeCode === nodeCode) {
            return {
              ...node,
              ...trackNode,
            }
          } else {
            return {
              ...node,
              flag: false,
              descriptions: [],
            }
          }
        })
        if (!isDeparture) {
          console.log(trackNode, nodeList)
        }
        const airline = airlines.find(
          airline =>
            airline.flightNO === flightNO &&
            airline.flightDate === airline.flightDate
        )
        if (airline) {
          const trackAirport = airline.airports.find(
            trackAirport => trackAirport.airport === airport
          )
          if (trackAirport) {
            trackAirport.trackSteps = trackAirport.trackSteps.map(node => {
              if (node.nodeCode === nodeCode) {
                node = {
                  ...node,
                  ...trackNode,
                }
              }
              return node
            })
          } else {
            airline.airports.push({
              airport,
              isDeparture: isDeparture,
              trackSteps: nodeList,
            })
          }
        } else {
          airlines.push({
            flightNO,
            flightDate,
            airports: [
              {
                airport,
                isDeparture: isDeparture,
                trackSteps: nodeList,
              },
            ],
          })
        }
        return airlines
      },
      [] as TrackAirline[]
    )
    trackAirlines.value = airlines.map(airline => {
      return {
        ...airline,
        airports:
          name.includes('Departure') === airline.airports[0].isDeparture
            ? airline.airports
            : airline.airports.reverse(),
      }
    })
  }

  watch(trackData, () => {
    getTrackAirlines()
  })

  // const getTrackAirlines = async () => {
  //   try {
  //     trackAirlines.value = await trackDataContentList.value.reduce(
  //       async (
  //         pre,
  //         { flightNO, airport, flightDate, isDeparture, dataContent }
  //       ) => {
  //         const airlines = await pre
  //         const trackNodeList = await getTrackData(
  //           DATACONTENT_ID[
  //             name.slice(0, 1).toLowerCase() + name.slice(1) + 'Track'
  //           ],
  //           dataContent
  //         )
  //         const trackSteps: TrackNode[] = trackNodesMap[
  //           isDeparture
  //             ? isInternational
  //               ? 'internationalDeparture'
  //               : 'departure'
  //             : isInternational
  //             ? 'internationalArrival'
  //             : 'arrival'
  //         ].map(node => {
  //           const sameNode = trackNodeList.find(
  //             ({ C0, C1, C2, C3 }) => C0 === node.nodeCode
  //           )
  //           if (sameNode) {
  //             node = {
  //               ...node,
  //               flag: true,
  //               descriptions: [
  //                 String(sameNode.C1 ?? ''),
  //                 String(sameNode.C2 ?? ''),
  //                 String(sameNode.C3 ?? ''),
  //               ],
  //             }
  //           } else {
  //             node = {
  //               ...node,
  //               flag: false,
  //               descriptions: [],
  //             }
  //           }
  //           return node
  //         })
  //         const trackAirport = {
  //           airport,
  //           isDeparture,
  //           trackSteps,
  //         }
  //         const airline = airlines.find(
  //           airline =>
  //             airline.flightNO === flightNO && airline.flightDate === flightDate
  //         )
  //         if (airline) {
  //           airline.airports.push(trackAirport)
  //         } else {
  //           airlines.push({
  //             flightNO,
  //             flightDate,
  //             airports: [trackAirport],
  //           })
  //         }
  //         return airlines
  //       },
  //       Promise.resolve([] as TrackAirline[])
  //     )
  //   } catch (error) {
  //     console.error(error)
  //   }
  // }
  // const getTrackData = async (id: number, dataContent: CommonValue[]) => {
  //   try {
  //     const {
  //       code,
  //       returnData: { listValues },
  //       message,
  //     } = await Query<TrackNodeData>({
  //       id,
  //       dataContent,
  //     })
  //     if (Number(code) !== 0) {
  //       throw new Error(message || '失败')
  //     }
  //     return listValues
  //   } catch (error) {
  //     return Promise.reject(error)
  //   }
  // }

  // const getSimulateTrackData = () => {
  //   const length = name.includes('Waybill') ? 2 : 1
  //   trackAirlines.value = Array.from({ length }, (_, airlineIndex) => {
  //     const length = name.includes('International') ? 1 : 2
  //     return {
  //       flightNO: airlineIndex < 1 ? 'CA1001' : 'CA1003',
  //       flightDate: '2022/09/10',
  //       airports: Array.from({ length }, (_, airportIndex) => ({
  //         airport: airportIndex < 1 ? '深圳机场' : '南京机场',
  //         isDeparture: name.includes('Departure')
  //           ? airportIndex < 1
  //             ? 1
  //             : 0
  //           : airportIndex < 1
  //           ? 0
  //           : 1,
  //         trackSteps: name.includes('International')
  //           ? name.includes('Departure')
  //             ? airlineIndex < 1
  //               ? simulateTrackNodesMap.internationalDeparture
  //               : simulateTrackNodesMap.internationalDeparture2
  //             : airlineIndex < 1
  //             ? simulateTrackNodesMap.internationalArrival
  //             : simulateTrackNodesMap.internationalArrival2
  //           : name.includes('Departure')
  //           ? airlineIndex < 1
  //             ? airportIndex < 1
  //               ? simulateTrackNodesMap.departure
  //               : simulateTrackNodesMap.arrival
  //             : airportIndex < 1
  //             ? simulateTrackNodesMap.departure2
  //             : simulateTrackNodesMap.arrival2
  //           : airlineIndex < 1
  //           ? airportIndex < 1
  //             ? simulateTrackNodesMap.arrival
  //             : simulateTrackNodesMap.departure
  //           : airportIndex < 1
  //           ? simulateTrackNodesMap.arrival2
  //           : simulateTrackNodesMap.departure2,
  //       })),
  //     }
  //   })
  // }
  // onMounted(() => {
  //   getSimulateTrackData()
  // })

  const trackBoxStyle = computed(
    () => (airports: TrackAirport[], index: number) => {
      const style: CSSProperties = {}
      const totalLength = airports.reduce((pre, current) => {
        return pre + current.trackSteps.length - 1
      }, 0)
      style.width = totalLength
        ? `calc((100% - ${airports.length - 1} * 8px) * ${
            (airports[index].trackSteps.length - 1) / totalLength
          })`
        : '100%'
      return style
    }
  )
  return {
    trackAirlines,
    trackBoxStyle,
  }
}
